/* Copyright (c) 2007 Sam Trenholme
 *
 * TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This software is provided 'as is' with no guarantees of correctness or
 * fitness for purpose.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>
#ifndef MINGW
#include <grp.h>
#endif /* MINGW */

/* Parameters that are currently hardcoded in the source code (This will
 * change once we get something very basic that works) */

#define DW_UID 99      /* UID that we run as */
#define DW_MINTIME 1178488417 /* Minimum allowed timestamp */

/* Make this look like a function just in case we need to make it
 * a function to make things thread-safe */
#define get_time() the_time

/* These are public functions */

/* Exit with a fatal error */
void dw_fatal(char *why);

/* Set the 64-bit timestamp starting at 290805600 unix() time (When
 * the Blake's 7 episode Gambit was originally broadcast); this should
 * be called once a second or so */
void set_time();

/* Read mararc parameters and set global variables based on those
 * parameters */
void process_mararc();

/* Initialize random number generator.  */
void init_rng();

/* Drop privileges and become unprivileged user */
void sandbox();

/* Get, from the Mararc parameters, the list of bind addresses
 * we will bind to; return this list as a comma-separated dw_str */
dw_str *get_bind_addrs();

/* Make sure a DNS packet is sane, and return the packet's
 * query ID.  If roy_arends_check has a value of 1, we also make sure
 * it's a DNS question (and not a reply).  This is named in honor of
 * Roy Arends, who pointed out the original MaraDNS didn't check this.
 * If this returns -1, the packet was not sane */
int32_t get_dns_qid(unsigned char *a, int len, int roy_arends_check);

/* Given a string with a DNS packet, and the length of that string,
 * make the first two butes of the packet (the query ID) the third
 * argument (qid), and return that random number. -1 on error */
int32_t set_dns_qid(unsigned char *packet, int len, uint16_t qid);

/* This function converts a dw_str object in to a null-terminated
 * C-string with the last item in the comma-separated list in the
 * dw_str, with any leading whitespace in the last item removed */
char *pop_last_item(dw_str *list);

/* This creates a netmask from a single number, storing the number in a
 * string of octets.  0 just makes the string all 0s; 1 makes the string
 * 0x80 0x00 0x00 etc.; 2 makes the string 0xc0 0x00 etc.; etc.
 * The string of octets has a length len (4 for ipv4; 16 for ipv6) */
void make_netmask(int num, uint8_t *str, int len);

